package jp.co.riiiver.sdk.sample

import android.Manifest.permission.*
import android.content.pm.PackageManager.PERMISSION_GRANTED
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.content.res.Configuration
import android.content.res.Configuration.SCREENLAYOUT_SIZE_LARGE
import android.graphics.Color
import android.graphics.Rect
import android.net.Uri
import android.os.Bundle
import android.os.Handler
import android.provider.Settings
import android.view.*
import android.widget.FrameLayout
import androidx.browser.customtabs.CustomTabsIntent
import androidx.fragment.app.Fragment
import com.riiiver.sdk.ersdk.ERApplet
import com.riiiver.sdk.ersdk.ERError
import com.riiiver.sdk.ersdk.preferences.ERPreferencesView
import com.riiiver.sdk.ersdk.preferences.ERPreferencesViewListener
import com.riiiver.sdk.ersdk.preferences.ERPreferencesViewPermissionListener
import jp.co.riiiver.sdk.sample.RiiiverSDKConfig.riiiverAuthUri
import kotlinx.android.synthetic.main.fragment_iiidea_setting.*
import kotlinx.android.synthetic.main.fragment_store.back
import timber.log.Timber
import java.io.File
import java.io.FileOutputStream
import java.net.URL

class IiideaSettingFragment : Fragment(), ERPreferencesViewListener,
    ERPreferencesViewPermissionListener {

    var PERMISSION_CODE = 1000

    private var isOnCreateRun: Boolean = false

    private lateinit var preferencesView: ERPreferencesView
    var iiidea: ERApplet? = null

    companion object {

        private var root = "preferences"

        private var activity: Activity? = null
        private lateinit var mChildOfContent: View
        private lateinit var frameLayoutParams: FrameLayout.LayoutParams
        private var usableHeightPrevious: Int = 0
        private var contentHeight: Int = 0
        private var isFirst = true

        fun initialize(context: Context) {
            Timber.d("initialize in: context=$context")
            // AssetからPreferences用WebコンテンツのPathを取得
            val paths = getAssetFilePaths(context, root)
            // アプリ配下にPreferences用Webコンテンツをコピー
            copyAssetFiles(context, paths)

            // mainActivityを保持
            activity = context as Activity

            // キーボードが重ならないように監視
            val content = activity!!.findViewById<View>(android.R.id.content) as FrameLayout
            mChildOfContent = content.getChildAt(0)
            mChildOfContent.viewTreeObserver.addOnGlobalLayoutListener {
                if (isFirst) {
                    contentHeight = mChildOfContent.height
                    isFirst = false
                }
                possiblyResizeChildOfContent()
            }
            frameLayoutParams = mChildOfContent.layoutParams as FrameLayout.LayoutParams
        }

        /**
         * 指定したアセットディレクトリの配下に属するファイルパスを全て取得する
         * @param context Context
         * @param assetDirectory アセット内のディレクトリ名
         * @return ファイルパスリスト
         */
        private fun getAssetFilePaths(context: Context, assetDirectory: String): ArrayList<String> {
            Timber.d("getAssetFilePaths in: context=$context assetDirectory=$assetDirectory")
            val ret: ArrayList<String> = arrayListOf()
            val pathList: Array<String>? = context.assets.list(assetDirectory)
            pathList?.forEach { path ->
                // 文字列検索
                if (Regex("\\.").containsMatchIn(path)) {
                    // "."が含まれている場合はファイルと判断する
                    ret.add(assetDirectory + File.separator + path)
                } else {
                    // "."が含まれていない場合はディレクトリと判断して配下のファイルを取得する
                    ret.addAll(getAssetFilePaths(context, assetDirectory + File.separator + path))
                }
            }
            return ret
        }

        /**
         * 指定したアセットディレクトリ配下のファイルをアプリ配下の領域にコピーする
         * @param context Context
         * @param filePaths コピーするファイル名のリスト(パス付き)
         */
        private fun copyAssetFiles(context: Context, filePaths: ArrayList<String>) {
            Timber.d("copyAssetFiles in: context=$context filePaths=$filePaths")
            filePaths.forEach { path ->
                val appPath = getRootPath(context) + path
                val file = File(appPath)
                val filePath = File(file.parent)
                if (!filePath.mkdirs()) {
                    Timber.w("mkdirs failure. already $filePath")
                }
                context.assets.open(path).use { inputStream ->
                    FileOutputStream(file, false).use { outputStream ->
                        inputStream.copyTo(outputStream)
                    }
                }
            }
        }

        /**
         * アプリ保存領域のRootディレクトリのパスを取得する
         * @param context Context
         * @return Rootディレクトリのパス
         */
        private fun getRootPath(context: Context): String {
            Timber.d("getRootPath in: context=$context")
            return context.filesDir.parent + File.separator
        }

        private fun possiblyResizeChildOfContent() {
            Timber.d("possiblyResizeChildOfContent in")
            val usableHeightNow = computeUsableHeight()
            if (usableHeightNow != usableHeightPrevious) {
                val usableHeightSansKeyboard = mChildOfContent.rootView.height
                val heightDifference = usableHeightSansKeyboard - usableHeightNow
                if (heightDifference > usableHeightSansKeyboard / 4) {
                    frameLayoutParams.height =
                        usableHeightSansKeyboard - heightDifference + getStatusBarHeight(activity!!) + getNavigationBarHeight(
                            activity!!
                        )
                } else {
                    frameLayoutParams.height = contentHeight
                }
                mChildOfContent.requestLayout()
                usableHeightPrevious = usableHeightNow
            }
        }

        private fun getStatusBarHeight(activity: Activity): Int {
            Timber.d("getStatusBarHeight in: activity=$activity")
            val rect = Rect()
            val window: Window = activity.window
            window.decorView.getWindowVisibleDisplayFrame(rect)
            return rect.top
        }

        private fun getNavigationBarHeight(context: Context): Int {
            Timber.d("getNavigationBarHeight in: context=$context")
            val result = 0
            val hasMenuKey = ViewConfiguration.get(context).hasPermanentMenuKey()
            val hasBackKey = KeyCharacterMap.deviceHasKey(KeyEvent.KEYCODE_BACK)

            if (!hasMenuKey && !hasBackKey) {
                val resources = context.resources
                val orientation = resources.configuration.orientation
                val resourceId = if (isTablet(context)) {
                    resources.getIdentifier(
                        if (orientation == Configuration.ORIENTATION_PORTRAIT) "navigation_bar_height" else "navigation_bar_height_landscape",
                        "dimen",
                        "android"
                    )
                } else {
                    resources.getIdentifier(
                        if (orientation == Configuration.ORIENTATION_PORTRAIT) "navigation_bar_height" else "navigation_bar_width",
                        "dimen",
                        "android"
                    )
                }

                if (resourceId > 0) {
                    return resources.getDimensionPixelSize(resourceId)
                }
            }
            return result
        }

        private fun isTablet(context: Context): Boolean {
            Timber.d("isTablet in: context=$context")
            return (context.resources.configuration.screenLayout and Configuration.SCREENLAYOUT_SIZE_MASK) >= SCREENLAYOUT_SIZE_LARGE
        }

        private fun computeUsableHeight(): Int {
            Timber.d("computeUsableHeight in")
            val r = Rect()
            mChildOfContent.getWindowVisibleDisplayFrame(r)
            return r.bottom - r.top
        }

    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(R.layout.fragment_iiidea_setting, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        Timber.d("in")
        back.setOnClickListener {
            Timber.d("back")
            back()
        }

        if (iiidea != null) {
            // PreferencesのViewを生成
            preferencesView = ERPreferencesView(context)

            // Preference画面読み込み
            preferencesView.loadPreferences(iiidea, URL("file://" + getRootPath(context!!) + root))

            preferencesView.preferencesListener = this
            preferencesView.permissionListener = this

            //背景画像設定
            preferencesView.setBackgroundColor(Color.BLACK)
            webPreference.addView(preferencesView)
        } else {
            Timber.e("applet is null.")
        }
    }

    override fun onResume() {
        super.onResume()
        Timber.d("onResume in")
        if (isOnCreateRun) {
            Timber.d("Ignore onResume which is called immediately.")
            // onCreate後、すぐに呼ばれるonResumeは無視する
            isOnCreateRun = false
            return
        }
        // フォアグラウンドになったらPreferencesを更新
        reloadPreferences()
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        Timber.d("onRequestPermissionsResult in: requestCode=$requestCode permissions=$permissions grantResults=$grantResults")
        if (requestCode == PERMISSION_CODE) {
            if (grantResults.isNotEmpty() && grantResults[0] == PERMISSION_GRANTED) {
                // パーミッションに変化がある場合はPreferencesViewをリロード
                reloadPreferences()
            }
        }
    }

    override fun updateTriggerPreferences(p0: ERApplet?) {
        Timber.d("updateTriggerPreferences in")
    }

    override fun updateServicePreferences(p0: ERApplet?) {
        Timber.d("updateServicePreferences in")
    }

    override fun updateActionPreferences(p0: ERApplet?) {
        Timber.d("updateActionPreferences in")
    }

    override fun requestPermissionScreen(permissionType: String?) {
        Timber.d("requestPermissionScreen in: permissionType =$permissionType")
        when (permissionType) {
            "location" -> {
                requestPermission(ACCESS_FINE_LOCATION)
            }
            "microphone" -> {
                requestPermission(RECORD_AUDIO)
            }
            "calendar" -> {
                requestPermission(READ_CALENDAR)
            }
            else -> {
                Timber.e("permissionType Error: permissionType=$permissionType")
            }
        }

    }

    override fun requestAuthScreen(blockId: String?) {
        Timber.d("requestAuthScreen in: blockId=$blockId")
        val uuid = RiiiverSDKManager.getUuid()
        showAuthenticationSession(riiiverAuthUri + "?blockId=" + blockId + "&token=Bearer%20" + RiiiverSDKManager.accessToken + "&uuid=" + uuid)
    }

    override fun requestDeleteAuthScreen(blockId: String?) {
        Timber.d("requestDeleteAuthScreen in")
        RiiiverSDKManager.deleteClientAuth(blockId!!) { erError: ERError? ->
            if (erError != null) {
                Timber.e("error %s", erError.message)
                return@deleteClientAuth
            }
        }
    }

    // パーミッション許可
    private fun requestPermission(permission: String) {
        Timber.d("requestPermission in: permission=$permission")
        if (this.shouldShowRequestPermissionRationale(permission)) {
            if (this.shouldShowRequestPermissionRationale(permission)) {
                this.requestPermissions(arrayOf(permission), PERMISSION_CODE)
            }
        } else {
            // 「今後は確認しない」にチェックが入っている場合
            Handler().post {
                // システムのアプリ設定画面
                val intent = Intent(
                    Settings.ACTION_APPLICATION_DETAILS_SETTINGS,
                    Uri.parse("package:" + this.activity!!.packageName)
                )
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                this.activity!!.startActivity(intent)
            }
        }
    }

    // 認証画面表示
    private fun showAuthenticationSession(url: String) {
        Timber.d("showAuthenticationSession in: url=$url")
        val builder = CustomTabsIntent.Builder()
        val customTabsIntent = builder.build()
        customTabsIntent.launchUrl(context, Uri.parse(url))
    }

    // Preferences画面更新
    private fun reloadPreferences() {
        Timber.d("reloadPreferences in")
        RiiiverSDKManager.getAWSToken { _: Boolean, _: Exception? ->
            preferencesView.reloadPreferences()
        }
    }

    private fun back() {
        fragmentManager!!.popBackStack()
    }
}